/* ============================================================================
 * File Name: SAR-Filter-VDAC_01c.c
 *
 * Description:
 *   Demo project showing ADC_SAR -> Filter -> VDAC data streaming using DMA
 *
 *
 * Note:
 *   1. This project also demonstrates an issue with ADC not performing correctly
 *      in the differential mode. The ADC output is always unsigned 0-2047 no matter
 *      if ADC_SAR in single or differential mode.
 *   2. The Filter expects input data as signed 16-bit, but receives only unsigned
 *      12-bit. Setting Filter Gain to a large value can compensate loss of the 
 *      signal amplitude to some degree, but not the offset.
 *
 * Note:
 *   This project uses several custom Community components and libraries,
 *   which can be omitted if an external signal generator is available.
 * 
 *   1. WaveGen8 (RAM-DMA-VDAC8 wave generator, DRAFT version is included in the project)
 *   2. DDS32: https://community.cypress.com/message/158566 (included in the project)  
 *   3. QuadDecoder_SW: https://community.cypress.com/thread/30654
 *   4. Annotation Library: https://community.cypress.com/thread/48049
 *
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/


#include <project.h>


#define BlinkLED(); {Pin_LED_Write(1); CyDelayUs(130); Pin_LED_Write(0);} // blink LED indicator

//===========================================
// Function prototypes
//===========================================
void DMA_Config(void);
void DMA2_Config(void);


void Init()
{   
    CYGlobalIntEnable;      // Enable Global Interrupts
    
    ADC_1_Start();
    //ADC_1_StartConvert(); // no need in triggered mode
    
    VDAC_1_Start();
    Opamp_1_Start();
    
    Filter_Start();
    
    DMA_Config();
    DMA2_Config();   
    
    //Sine genrator with rotary encoder->
    WaveGen_1_CreateSineWave_B(255, 0); // fill Buffer with default values (ampl, phase) 
    WaveGen_1_Start();                  // start wave generator
    VDAC_2_Start();
    Opamp_2_Start();

    
    Decoder_1_Start();      // use rotary encoder to change sine frequency      
    DDS_1_Start();
    DDS_1_SetFrequency(Decoder_1_Position);
}

int main()
{
    Init();

    for(;;)
    {
        
        if (Decoder_1_PositionChanged)              // encoder position changed
        { 
            DDS_1_SetFrequency(Decoder_1_Position);
            //BlinkLED();   // debugging...
        }
        
        if (Decoder_1_BtnPressed)              // encoder position changed
        { 
            Decoder_1_SetPosition(Decoder_1_StartPosition);
            //BlinkLED();   // debugging...
        }
        
    }
    
} /* End of main */


//======================================================================================
// Configure DMA transfer from ADC_SAR to Filter 
// DMA has single TD:
// TD[0] transfers 2 bytes (uint16) data from ADC to Filter
//======================================================================================
void DMA_Config(void)
{
    #define BYTES_PER_BURST          (2u)
    #define REQUEST_PER_BURST        (1u)
    #define UPPER_SRC_ADDRESS        CYDEV_PERIPH_BASE
    #define UPPER_DEST_ADDRESS       CYDEV_PERIPH_BASE
    
    
    // Declare variable to hold the handle for DMA channel /
    uint8 channelHandle;

    // Declare DMA Transaction Descriptor for memory transfer into Filter Channel./
    uint8 tdChanA;

    // Configure the DMA to Transfer the data in 1 burst with individual trigger for each burst.
    channelHandle = DMA_DmaInitialize(BYTES_PER_BURST, REQUEST_PER_BURST, HI16(UPPER_SRC_ADDRESS), HI16(UPPER_DEST_ADDRESS));

    // This function allocates a TD for use with an initialized DMA channel
    tdChanA = CyDmaTdAllocate();

    // Configure the tdChanA to transfer 1 byte with no next TD
    CyDmaTdSetConfiguration(tdChanA, 2u, DMA_INVALID_TD, 0u);

    // Set the source address as ADC_DelSig and the destination as Filter Channel A
    CyDmaTdSetAddress(tdChanA, LO16((uint32)ADC_1_SAR_WRK0_PTR), LO16((uint32)Filter_STAGEA_PTR)); //SAR_ADC

    // Set tdChanA to be the initial TD associated with channelHandle 
    CyDmaChSetInitialTd(channelHandle, tdChanA);

    // Enable the DMA channel represented by channelHandle and preserve the TD
    CyDmaChEnable(channelHandle, 1u);
    
    //Filter_COHER_REG = 0x55;
    
    #define FILTER_DATA_ALIGN   (0x05u)
    
    // For 9-16 bits filter resolution, Filter coherency should be MID, Dalign should be enabled
	Filter_SetCoherency(Filter_CHANNEL_A, Filter_KEY_MID );
    Filter_DALIGN_REG =  Filter_DALIGN_REG | FILTER_DATA_ALIGN;
}


//======================================================================================
// Configure DMA2 transfer from Filter to VDAC8 
// DMA_1 has single TD:
// TD[0] transfers signed (int8) data from Filter to VDAC8
//======================================================================================
void DMA2_Config(void)
{
    uint8 DMA2_Chan;
    uint8 DMA2_TD[1];

    // DMA Configuration 
    #define DMA2_BYTES_PER_BURST    (1u)
    #define DMA2_REQUEST_PER_BURST  (1u)
    #define TRANSFER_COUNT          (1u)
    #define DMA2_SRC_BASE           (CYDEV_PERIPH_BASE)
    #define DMA2_DST_BASE           (CYDEV_PERIPH_BASE)

    DMA2_Chan = DMA2_DmaInitialize(DMA2_BYTES_PER_BURST, DMA2_REQUEST_PER_BURST, HI16(DMA2_SRC_BASE), HI16(DMA2_DST_BASE));
    DMA2_TD[0] = CyDmaTdAllocate();
    
    CyDmaTdSetConfiguration(DMA2_TD[0], TRANSFER_COUNT, DMA2_TD[0], 0);

    CyDmaTdSetAddress(DMA2_TD[0], LO16((uint32)Filter_HOLDAM_PTR), LO16((uint32)VDAC_1_Data_PTR)); //Filter->VDAC

    CyDmaChSetInitialTd(DMA2_Chan, DMA2_TD[0]);
    CyDmaChEnable(DMA2_Chan, 1);
}



/* [] END OF FILE */
